#include "attachment_file.h"

#include <stdio.h>
#include <magic.h>
#include <string.h>
#include <sys/mman.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

struct attachment *attachment_file_create(char *filename)
{
	magic_t m;
	const char *type;
	char *basetype, *subtype;
	size_t i;
	struct attachment *a;
	int fd;
	char *name;
	
	name = filename;
	for (i = 0; i < strlen(filename); i++)
		if (filename[i] == '/')
			name = filename + i + 1;
	
	m = magic_open(MAGIC_MIME_TYPE);
	if (m == NULL) {
		return NULL;
	}
	
	magic_load(m, NULL);
	
	type = magic_file(m, filename);
	if (!type) {
		printf("%s\n", magic_error(m));
	}
	
	for (i = 0; i < strlen(type); i++) {
		if (type[i] == '/') {
			basetype = malloc(i+1);
			memcpy(basetype, type, i);
			basetype[i] = 0;
			subtype = malloc(strlen(type + i));
			strcpy(subtype, type + i + 1);
		}
	}
	
	a = attachment_alloc();
	attachment_basetype_set(a, basetype);
	attachment_type_set(a, subtype);

	fd = open(filename, O_RDONLY);
	if (fd >= 0) {
		void * mem;
		struct stat stat_info;
		int r;
		
		r = fstat(fd, &stat_info);
		if (r >= 0) {
			mem = mmap(NULL, stat_info.st_size, 
			    PROT_READ, MAP_PRIVATE,
			    fd, 0);
			if (mem == MAP_FAILED)
				goto err_mmap;
			attachment_data_set(a, 
			    mem, stat_info.st_size);
		} else {
			goto err_mmap;
		}
	}
	attachment_name_set(a, name);
	
	
	goto out_ok;
err_mmap:
	attachment_free(a);
	a = NULL;
out_ok:
	close(fd);
	free(basetype);
	free(subtype);
	
	magic_close(m);
	
	return a;
}

